@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2007 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2007 Laszlo Molnar
+   Copyright (C) 1996-2009 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2009 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -22,7 +22,7 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    Markus F.X.J. Oberhumer              Laszlo Molnar
-   <mfx@users.sourceforge.net>          <ml1050@users.sourceforge.net>
+   <markus@oberhumer.com>               <ml1050@users.sourceforge.net>
  */
 
 
@@ -78,10 +78,13 @@
 #  if (ACC_OS_WIN32 || ACC_OS_WIN64)
 #  elif defined(__linux__)
 #    pragma warning(error: 424)         // #424: extra ";" ignored
+//#    pragma warning(disable: 128)       // #128: loop is not reachable from preceding code
+//#    pragma warning(disable: 181)       // #181: argument is incompatible with corresponding format string conversion
 #    pragma warning(disable: 193)       // #193: zero used for undefined preprocessing identifier
 #    pragma warning(disable: 810)       // #810: conversion from "A" to "B" may lose significant bits
 #    pragma warning(disable: 981)       // #981: operands are evaluated in unspecified order
-#    pragma warning(disable: 1418)      // #1418: external definition with no prior declaration
+#    pragma warning(disable: 1418)      // #1418: external function definition with no prior declaration
+//#    pragma warning(disable: 1419)      // #1419: external declaration in primary source file
 #  else
 #    error "untested platform"
 #  endif
@@ -107,10 +110,13 @@
 #elif (ACC_CC_SUNPROC)
 //#  pragma error_messages(off,"badargtype2w")    // FIXME
 #elif (ACC_CC_WATCOMC)
-#  if (__WATCOMC__ < 1100)
-#    error "need Watcom C++ 11.0c or newer"
+#  if (__WATCOMC__ < 1280)
+#    error "need Open Watcom C++ 1.8 or newer"  // because earlier versions do not support nested classes
 #  endif
 #  if defined(__cplusplus)
+#    pragma warning 367 9               // w3: conditional expression in if statement is always true
+#    pragma warning 368 9               // w3: conditional expression in if statement is always false
+#    pragma warning 389 9               // w3: integral value may be truncated
 #    pragma warning 656 9               // w5: define this function inside its class definition (may improve code quality)
 #  endif
 #endif
@@ -160,6 +166,9 @@
 #if 0 && !defined(WITH_LZMA)
 #  define WITH_LZMA 1
 #endif
+#if 1 && (ACC_CC_WATCOMC)
+#  undef WITH_LZMA
+#endif
 #if defined(UPX_OFFICIAL_BUILD)
 #  if !defined(WITH_LZMA) || !defined(WITH_NRV) || !defined(WITH_UCL)
 #    error
@@ -301,16 +310,18 @@
 #endif
 
 
-#undef __attribute_packed
-#if (ACC_CC_GNUC || ACC_CC_INTELC || ACC_CC_PATHSCALE)
-#  if (ACC_ARCH_I386) && (ACC_CC_INTELC && (__INTEL_COMPILER < 800))
-#  elif (0 && (ACC_ARCH_AMD64 || ACC_ARCH_I386))
-#  else
-#    define __attribute_packed      __attribute__((__packed__,__aligned__(1)))
-#  endif
+#if (ACC_CC_INTELC && (__INTEL_COMPILER < 800))
+#elif (0 && (ACC_ARCH_AMD64 || ACC_ARCH_I386))
+#elif (ACC_CC_GNUC || ACC_CC_INTELC || ACC_CC_PATHSCALE)
+#  define __packed_struct(s)        struct s {
+#  define __packed_struct_end()     } __attribute__((__packed__,__aligned__(1)));
+#elif (ACC_CC_WATCOMC)
+#  define __packed_struct(s)        _Packed struct s {
+#  define __packed_struct_end()     };
 #endif
-#if !defined(__attribute_packed)
-#  define __attribute_packed
+#if !defined(__packed_struct)
+#  define __packed_struct(s)        struct s {
+#  define __packed_struct_end()     };
 #endif
 
 
@@ -323,8 +334,9 @@
 
 #define __COMPILE_TIME_ASSERT_ALIGNOF_SIZEOF(a,b) { \
      typedef a acc_tmp_a_t; typedef b acc_tmp_b_t; \
-     struct acc_tmp_t { acc_tmp_b_t x; acc_tmp_a_t y; acc_tmp_b_t z; } __attribute_packed; \
+     __packed_struct(acc_tmp_t) acc_tmp_b_t x; acc_tmp_a_t y; acc_tmp_b_t z; __packed_struct_end() \
      COMPILE_TIME_ASSERT(sizeof(struct acc_tmp_t) == 2*sizeof(b)+sizeof(a)) \
+     COMPILE_TIME_ASSERT(sizeof(((acc_tmp_t*)0)->x)+sizeof(((acc_tmp_t*)0)->y)+sizeof(((acc_tmp_t*)0)->z) == 2*sizeof(b)+sizeof(a)) \
    }
 #if defined(__acc_alignof)
 #  define __COMPILE_TIME_ASSERT_ALIGNOF(a,b) \
@@ -379,12 +391,14 @@ inline unsigned UPX_MIN(unsigned a, unsigned b) { return a < b ? a : b; }
 
 #define ByteArray(var, size)    Array(unsigned char, var, size)
 
-struct nocopy
+struct noncopyable
 {
-    inline nocopy() {}
+protected:
+    inline noncopyable() {}
+    inline ~noncopyable() {}
 private:
-    nocopy(const nocopy &); // undefined
-    nocopy& operator=(const nocopy &); // undefined
+    noncopyable(const noncopyable &); // undefined
+    const noncopyable& operator=(const noncopyable &); // undefined
 };
 
 
@@ -457,6 +471,12 @@ private:
 #define UPX_F_VMLINUX_ARMEL     28
 #define UPX_F_MACH_i386         29
 #define UPX_F_LINUX_ELF32_MIPSEL 30
+#define UPX_F_VMLINUZ_ARMEL     31
+#define UPX_F_MACH_ARMEL        32
+
+#define UPX_F_DYLIB_i386        33
+#define UPX_F_MACH_AMD64        34
+#define UPX_F_DYLIB_AMD64       35
 
 #define UPX_F_PLAIN_TEXT        127
 
@@ -468,6 +488,8 @@ private:
 #define UPX_F_MACH_FAT          134
 #define UPX_F_VMLINUX_ARMEB     135
 #define UPX_F_VMLINUX_PPC32     136
+#define UPX_F_LINUX_ELF32_MIPSEB 137
+#define UPX_F_DYLIB_PPC32       138
 
 
 // compression methods
