@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2007 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2007 Laszlo Molnar
+   Copyright (C) 1996-2009 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2009 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -21,8 +21,8 @@
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-   Markus F.X.J. Oberhumer   Laszlo Molnar
-   markus@oberhumer.com      ml1050@users.sourceforge.net
+   Markus F.X.J. Oberhumer              Laszlo Molnar
+   <markus@oberhumer.com>               <ml1050@users.sourceforge.net>
  */
 
 
@@ -34,11 +34,11 @@
 // ElfLinker
 **************************************************************************/
 
-class ElfLinker : private nocopy
+class ElfLinker : private noncopyable
 {
     friend class Packer;
 public:
-    const N_BELE_RTP::AbstractPolicy *bele;
+    const N_BELE_RTP::AbstractPolicy *bele; // target endianness
 protected:
     struct      Section;
     struct      Symbol;
@@ -107,10 +107,18 @@ protected:
     virtual void relocate();
     virtual void relocate1(const Relocation *, upx_byte *location,
                            unsigned value, const char *type);
+
+    // target endianness abstraction
+    unsigned get_te16(const void *p)        const { return bele->get16(p); }
+    unsigned get_te32(const void *p)        const { return bele->get32(p); }
+    acc_uint64l_t get_te64(const void *p)   const { return bele->get64(p); }
+    void set_te16(void *p, unsigned v)      const { bele->set16(p, v); }
+    void set_te32(void *p, unsigned v)      const { bele->set32(p, v); }
+    void set_te64(void *p, acc_uint64l_t v) const { bele->set64(p, v); }
 };
 
 
-struct ElfLinker::Section : private nocopy
+struct ElfLinker::Section : private noncopyable
 {
     char *name;
     void *input;
@@ -125,7 +133,7 @@ struct ElfLinker::Section : private nocopy
 };
 
 
-struct ElfLinker::Symbol : private nocopy
+struct ElfLinker::Symbol : private noncopyable
 {
     char *name;
     Section *section;
@@ -136,7 +144,7 @@ struct ElfLinker::Symbol : private nocopy
 };
 
 
-struct ElfLinker::Relocation : private nocopy
+struct ElfLinker::Relocation : private noncopyable
 {
     const Section *section;
     unsigned offset;
@@ -195,6 +203,17 @@ protected:
 };
 
 
+class ElfLinkerMipsBE : public ElfLinker
+{
+    typedef ElfLinker super;
+public:
+    ElfLinkerMipsBE() { bele = &N_BELE_RTP::be_policy; }
+protected:
+    virtual void relocate1(const Relocation *, upx_byte *location,
+                           unsigned value, const char *type);
+};
+
+
 class ElfLinkerMipsLE : public ElfLinker
 {
     typedef ElfLinker super;
