@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2007 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2007 Laszlo Molnar
+   Copyright (C) 1996-2009 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2009 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -22,7 +22,7 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    Markus F.X.J. Oberhumer              Laszlo Molnar
-   <mfx@users.sourceforge.net>          <ml1050@users.sourceforge.net>
+   <markus@oberhumer.com>               <ml1050@users.sourceforge.net>
  */
 
 
@@ -45,6 +45,11 @@ enum {
 struct options_t {
     int cmd;
 
+    // mp (meta) options
+    int mp_compress_task;
+    bool mp_query_format;
+    bool mp_query_num_tasks;
+
     // compression options
     int method;
     bool method_lzma_seen;
@@ -129,7 +134,7 @@ struct options_t {
         bool boot_only;
         bool no_align;
         bool do_8bit;
-        bool do_8mb;
+        bool do_8mib;
     } ps1_exe;
     struct {
         unsigned blocksize;
