@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2007 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2007 Laszlo Molnar
+   Copyright (C) 1996-2009 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2009 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -22,7 +22,7 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    Markus F.X.J. Oberhumer              Laszlo Molnar
-   <mfx@users.sourceforge.net>          <ml1050@users.sourceforge.net>
+   <markus@oberhumer.com>               <ml1050@users.sourceforge.net>
  */
 
 
@@ -148,14 +148,12 @@ Linker* PackArmPe::newLinker() const
 // import handling
 **************************************************************************/
 
-struct import_desc
-{
+__packed_struct(import_desc)
     LE32  oft;      // orig first thunk
     char  _[8];
     LE32  dllname;
     LE32  iat;      // import address table
-}
-__attribute_packed;
+__packed_struct_end()
 
 void PackArmPe::processImports(unsigned myimport, unsigned iat_off) // pass 2
 {
@@ -714,7 +712,7 @@ void PackArmPe::pack(OutputFile *fo)
 
     // limit stack size needed for runtime decompression
     upx_compress_config_t cconf; cconf.reset();
-    cconf.conf_lzma.max_num_probs = 1846 + (768 << 4); // ushort: ~28KB stack
+    cconf.conf_lzma.max_num_probs = 1846 + (768 << 4); // ushort: ~28 KiB stack
     compressWithFilters(&ft, 2048, &cconf, filter_strategy,
                         ih.codebase, rvamin, 0, NULL, 0);
 // info: see buildLoader()
@@ -898,7 +896,7 @@ void PackArmPe::pack(OutputFile *fo)
     //    set_le32(ibuf + ic,get_le32("UPX "));
     ibuf.clear(0, oh.filealign);
 
-    info("Image size change: %u -> %u kBytes",
+    info("Image size change: %u -> %u KiB",
          ih.imagesize / 1024, oh.imagesize / 1024);
 
     infoHeader("[Writing compressed file]");
