@@ -2,9 +2,9 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2007 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2007 Laszlo Molnar
-   Copyright (C) 2001-2007 John F. Reiser
+   Copyright (C) 1996-2009 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2009 Laszlo Molnar
+   Copyright (C) 2001-2009 John F. Reiser
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -23,7 +23,7 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    Markus F.X.J. Oberhumer              Laszlo Molnar
-   <mfx@users.sourceforge.net>          <ml1050@users.sourceforge.net>
+   <markus@oberhumer.com>               <ml1050@users.sourceforge.net>
 
    John F. Reiser
    <jreiser@users.sourceforge.net>
@@ -132,8 +132,8 @@ PackLinuxI386::generateElfHdr(
     unsigned const brka
 )
 {
-    cprElfHdr2 *const h2 = (cprElfHdr2 *)&elfout;
-    cprElfHdr3 *const h3 = (cprElfHdr3 *)&elfout;
+    cprElfHdr2 *const h2 = (cprElfHdr2 *)(void *)&elfout;
+    cprElfHdr3 *const h3 = (cprElfHdr3 *)(void *)&elfout;
     memcpy(h3, proto, sizeof(*h3));  // reads beyond, but OK
 
     assert(h2->ehdr.e_phoff     == sizeof(Elf32_Ehdr));
@@ -526,28 +526,31 @@ bool PackLinuxI386::canPack()
                 ei_osabi = osabi0;  // Proudly declares its osabi in Ehdr.
                 break;
             default:
-            unsigned const e_phnum = get_native16(&ehdr.e_phnum);
+            unsigned const e_phnum = get_te16(&ehdr.e_phnum);
             if (e_phnum<=(512/sizeof(Elf32_Phdr))) {
-                char buf2[512];
-                fi->seek(get_native32(&ehdr.e_phoff), SEEK_SET);
-                fi->readx(buf2, sizeof(buf2));
+                union {
+                    unsigned char buf2[512];
+                    //Elf32_Phdr phdr;
+                } u;
+                fi->seek(get_te32(&ehdr.e_phoff), SEEK_SET);
+                fi->readx(u.buf2, sizeof(u.buf2));
                 fi->seek(0, SEEK_SET);
-                Elf32_Phdr const *phdr = (Elf32_Phdr const *)buf2;
+                Elf32_Phdr const *phdr = (Elf32_Phdr *) u.buf2;
                 for (unsigned j=0; j < e_phnum; ++phdr, ++j) {
-                    if (phdr->PT_NOTE == get_native32(&phdr->p_type)) {
-                        unsigned const offset = get_native32(&phdr->p_offset);
+                    if (phdr->PT_NOTE == get_te32(&phdr->p_type)) {
+                        unsigned const offset = get_te32(&phdr->p_offset);
                         struct Elf32_Note note; memset(&note, 0, sizeof(note));
                         fi->seek(offset, SEEK_SET);
                         fi->readx(&note, sizeof(note));
                         fi->seek(0, SEEK_SET);
-                        if (4==get_native32(&note.descsz)
-                        &&  1==get_native32(&note.type)
+                        if (4==get_te32(&note.descsz)
+                        &&  1==get_te32(&note.type)
                         &&  0==note.end ) {
-                            if (0==strcmp("NetBSD", (char const *)&note.text)) {
+                            if (0==strcmp("NetBSD", (char const *)note.text)) {
                                 ei_osabi = Elf32_Ehdr::ELFOSABI_NETBSD;
                                 break;
                             }
-                            if (0==strcmp("OpenBSD", (char const *)&note.text)) {
+                            if (0==strcmp("OpenBSD", (char const *)note.text)) {
                                 ei_osabi = Elf32_Ehdr::ELFOSABI_OPENBSD;
                                 break;
                             }
