@@ -2,9 +2,9 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 2004-2007 John Reiser
-   Copyright (C) 1996-2007 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2007 Laszlo Molnar
+   Copyright (C) 2004-2009 John Reiser
+   Copyright (C) 1996-2009 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2009 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -22,11 +22,11 @@
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-   Markus F.X.J. Oberhumer   Laszlo Molnar
-   markus@oberhumer.com      ml1050@users.sourceforge.net
+   Markus F.X.J. Oberhumer              Laszlo Molnar
+   <markus@oberhumer.com>               <ml1050@users.sourceforge.net>
 
    John Reiser
-   jreiser@users.sourceforge.net
+   <jreiser@users.sourceforge.net>
  */
 
 
@@ -64,7 +64,7 @@ PackVmlinuxBase<T>::PackVmlinuxBase(InputFile *f,
     n_ptload(0), phdri(NULL), shdri(NULL), shstrtab(NULL)
 {
     ElfClass::compileTimeAssertions();
-    bele = N_BELE_CTP::getRTP<BeLePolicy>();
+    bele = N_BELE_CTP::getRTP((const BeLePolicy*) NULL);
 }
 
 template <class T>
@@ -212,7 +212,7 @@ void PackVmlinuxBase<T>::pack(OutputFile *fo)
 {
     unsigned fo_off = 0;
     Ehdr ehdro;
-    U32 tmp_u32;
+    TE32 tmp_u32;
 
     // NULL
     // .text(PT_LOADs) .note(1st page) .note(rest)
@@ -269,7 +269,7 @@ void PackVmlinuxBase<T>::pack(OutputFile *fo)
 
     upx_compress_config_t cconf; cconf.reset();
     // limit stack size needed for runtime decompression
-    cconf.conf_lzma.max_num_probs = 1846 + (768 << 4); // ushort: ~28KB stack
+    cconf.conf_lzma.max_num_probs = 1846 + (768 << 4); // ushort: ~28 KiB stack
 
     unsigned ppc32_extra = 0;
     if (Ehdr::EM_PPC==my_e_machine) {
@@ -301,7 +301,7 @@ void PackVmlinuxBase<T>::pack(OutputFile *fo)
         if (UPX_E_OK!=r || len_unc<=len_cpr)  // FIXME: allow no compression
             throwInternalError("Ehdr compression failed");
 
-        struct b_info { // 12-byte header before each compressed block
+        __packed_struct(b_info) // 12-byte header before each compressed block
             unsigned sz_unc;  // uncompressed_size
             unsigned sz_cpr;  //   compressed_size
             unsigned char b_method;  // compression algorithm
@@ -309,8 +309,7 @@ void PackVmlinuxBase<T>::pack(OutputFile *fo)
             unsigned char b_cto8;  // filter parameter
             unsigned char b_unused;  // FIXME: !=0  for partial-block unfilter
             // unsigned f_offset, f_len;  // only if    partial-block unfilter
-        }
-        __attribute_packed;
+        __packed_struct_end()
 
         struct b_info hdr_info;
         set_be32(&hdr_info.sz_unc, len_unc);
@@ -376,102 +375,102 @@ void PackVmlinuxBase<T>::pack(OutputFile *fo)
 
 #if 0  /*{ Documentation: changes to arch/powerpc/boot/main.c */
 struct b_info {
-	unsigned sz_unc;
-	unsigned sz_cpr;
-	unsigned char b_method;
-	unsigned char b_ftid;
-	unsigned char b_cto8;
-	unsigned char b_unused;
+    unsigned sz_unc;
+    unsigned sz_cpr;
+    unsigned char b_method;
+    unsigned char b_ftid;
+    unsigned char b_cto8;
+    unsigned char b_unused;
 };
 typedef int (*upx_f_unc)(  /* uncompress */
-	unsigned char const *src, unsigned  src_len,
-	unsigned char       *dst, unsigned *dst_len,
-	int method
+    unsigned char const *src, unsigned  src_len,
+    unsigned char       *dst, unsigned *dst_len,
+    int method
 );
 typedef int (*upx_f_unf)(  /* unfilter */
-	unsigned char *buf,
-	unsigned len,
-	unsigned cto8,
-	unsigned ftid
+    unsigned char *buf,
+    unsigned len,
+    unsigned cto8,
+    unsigned ftid
 );
 
 unsigned char const *upx_expand(  // return updated src [aligned, too]
-	unsigned char const *src,
-	unsigned char       *dst,
-	unsigned *const dst_len,
-	upx_f_unc const f_unc
+    unsigned char const *src,
+    unsigned char       *dst,
+    unsigned *const dst_len,
+    upx_f_unc const f_unc
 )
 {
-	if (0==*dst_len) {
-		*dst_len = ((struct b_info const *)src)->sz_unc;
-	}
-	upx_f_unf const f_unf = (upx_f_unf)(sizeof(int) + (char *)f_unc);
-	unsigned total_len = 0;
-	unsigned need = *dst_len;
-	while (0 < need) {
-		struct b_info const *const b_hdr = (struct b_info const *)src;
-		src = (unsigned char const *)(1+ b_hdr);
-		unsigned blk_len = b_hdr->sz_unc;
-		int const rv = (*f_unc)(src, b_hdr->sz_cpr, dst, &blk_len, b_hdr->b_method);
-		if (0!=rv) {
-			printf("decompress error %d\n\r", rv);
-			exit();
-		}
-		if (b_hdr->b_ftid) {
-			(*f_unf)(dst, b_hdr->sz_unc, b_hdr->b_cto8, b_hdr->b_ftid);
-		}
-		src += b_hdr->sz_cpr;
-		src += (3& -(int)src);  // 4-byte align
-		dst += b_hdr->sz_unc;
-		total_len += b_hdr->sz_unc;
-		need -= b_hdr->sz_unc;
-	}
-	*dst_len = total_len;
-	return src;
+    if (0==*dst_len) {
+        *dst_len = ((struct b_info const *)src)->sz_unc;
+    }
+    upx_f_unf const f_unf = (upx_f_unf)(sizeof(int) + (char *)f_unc);
+    unsigned total_len = 0;
+    unsigned need = *dst_len;
+    while (0 < need) {
+        struct b_info const *const b_hdr = (struct b_info const *)src;
+        src = (unsigned char const *)(1+ b_hdr);
+        unsigned blk_len = b_hdr->sz_unc;
+        int const rv = (*f_unc)(src, b_hdr->sz_cpr, dst, &blk_len, b_hdr->b_method);
+        if (0!=rv) {
+            printf("decompress error %d\n\r", rv);
+            exit();
+        }
+        if (b_hdr->b_ftid) {
+            (*f_unf)(dst, b_hdr->sz_unc, b_hdr->b_cto8, b_hdr->b_ftid);
+        }
+        src += b_hdr->sz_cpr;
+        src += (3& -(int)src);  // 4-byte align
+        dst += b_hdr->sz_unc;
+        total_len += b_hdr->sz_unc;
+        need -= b_hdr->sz_unc;
+    }
+    *dst_len = total_len;
+    return src;
 }
 
 void prep_kernel(unsigned long a1, unsigned long a2)
 {
-	upx_f_unc f_unc = (upx_f_unc)0;
+    upx_f_unc f_unc = (upx_f_unc)0;
 
-	vmlinuz.addr = (unsigned long)_vmlinux_start;
-	vmlinuz.size = (unsigned long)(_vmlinux_end - _vmlinux_start);
+    vmlinuz.addr = (unsigned long)_vmlinux_start;
+    vmlinuz.size = (unsigned long)(_vmlinux_end - _vmlinux_start);
 
-	if (0x55505821 == *(unsigned *)vmlinuz.addr) {
-		/* was compressed by upx */
-		/* Uncompress elfheader */
-		unsigned const *const fwa = (unsigned *)vmlinuz.addr;
-		f_unc = (upx_f_unc)((char *)&fwa[2] + fwa[1]);
-		unsigned dst_len = 0;
-		unsigned char const *src = (unsigned char const *)&fwa[2];
-		src = upx_expand(src, (unsigned char *)elfheader, &dst_len, f_unc);
-		unsigned const src_len = src - (unsigned char const *)&fwa[0];
-		vmlinuz.addr += src_len;
-		vmlinuz.size -= src_len;
-	} else {
-		memcpy(elfheader, (const void *)vmlinuz.addr,
-		       sizeof(elfheader));
-	}
+    if (0x55505821 == *(unsigned *)vmlinuz.addr) {
+        /* was compressed by upx */
+        /* Uncompress elfheader */
+        unsigned const *const fwa = (unsigned *)vmlinuz.addr;
+        f_unc = (upx_f_unc)((char *)&fwa[2] + fwa[1]);
+        unsigned dst_len = 0;
+        unsigned char const *src = (unsigned char const *)&fwa[2];
+        src = upx_expand(src, (unsigned char *)elfheader, &dst_len, f_unc);
+        unsigned const src_len = src - (unsigned char const *)&fwa[0];
+        vmlinuz.addr += src_len;
+        vmlinuz.size -= src_len;
+    } else {
+        memcpy(elfheader, (const void *)vmlinuz.addr,
+               sizeof(elfheader));
+    }
 =====
-	/* Eventually gunzip the kernel */
-	if (f_unc) {
-		/* was compressed by upx */
-		printf("upx_expand (0x%lx <- 0x%lx:0x%0lx)...",
-		       vmlinux.addr, vmlinuz.addr, vmlinuz.addr+vmlinuz.size);
-		unsigned dst_len = vmlinux.size - elfoffset;
-		upx_expand((unsigned char const *)vmlinuz.addr,
-		           (unsigned char       *)vmlinux.addr, &dst_len, f_unc);
-		printf("done 0x%lx bytes\n\r", len);
-	} else {
-		memmove((void *)vmlinux.addr,(void *)vmlinuz.addr,
-			vmlinuz.size);
-		/* Skip over the ELF header */
+    /* Eventually gunzip the kernel */
+    if (f_unc) {
+        /* was compressed by upx */
+        printf("upx_expand (0x%lx <- 0x%lx:0x%0lx)...",
+               vmlinux.addr, vmlinuz.addr, vmlinuz.addr+vmlinuz.size);
+        unsigned dst_len = vmlinux.size - elfoffset;
+        upx_expand((unsigned char const *)vmlinuz.addr,
+                   (unsigned char       *)vmlinux.addr, &dst_len, f_unc);
+        printf("done 0x%lx bytes\n\r", len);
+    } else {
+        memmove((void *)vmlinux.addr,(void *)vmlinuz.addr,
+            vmlinuz.size);
+        /* Skip over the ELF header */
 #ifdef DEBUG
-		printf("... skipping 0x%lx bytes of ELF header\n\r",
-				elfoffset);
+        printf("... skipping 0x%lx bytes of ELF header\n\r",
+                elfoffset);
 #endif
-		vmlinux.addr += elfoffset;
-	}
+        vmlinux.addr += elfoffset;
+    }
 =====
 }
 #endif  /*}*/
@@ -485,9 +484,7 @@ void prep_kernel(unsigned long a1, unsigned long a2)
     const unsigned lsize = getLoaderSize();
 
     defineDecompressorSymbols();
-    if (ft.id!=0) {
-        defineFilterSymbols(&ft);
-    }
+    defineFilterSymbols(&ft);
     relocateLoader();
 
     MemBuffer loader(lsize);
@@ -630,7 +627,7 @@ void prep_kernel(unsigned long a1, unsigned long a2)
 
     fo->seek(0, SEEK_SET);
     fo->write(&ehdro, sizeof(ehdro));
-    fo->write(&shdro, sizeof(shdro));
+    fo->write(&shdro[0], sizeof(shdro));
     if (Ehdr::EM_PPC==my_e_machine) {
         fo->seek(sizeof(unsigned), SEEK_CUR);
         set_be32(&ppc32_extra, ppc32_extra - 2*sizeof(unsigned) + txt_c_len);
@@ -711,7 +708,7 @@ int PackVmlinuxBase<T>::canUnpack()
 template <class T>
 void PackVmlinuxBase<T>::unpack(OutputFile *fo)
 {
-    U32 word;
+    TE32 word;
     PackHeader const ph_tmp(ph);
 
     fi->seek(p_note0->sh_offset, SEEK_SET);
@@ -839,7 +836,7 @@ const int *PackVmlinuxPPC32::getFilters() const
 
 bool PackVmlinuxI386::is_valid_e_entry(Addr e_entry)
 {
-    return 0==(0x000fffff & e_entry); // entry on whole 1MB
+    return 0==(0x000fffff & e_entry); // entry on whole 1 MiB
 }
 
 
@@ -1017,7 +1014,7 @@ unsigned PackVmlinuxI386::write_vmlinux_head(
     // COMPRESSED_LENGTH
     fo->write(&stub_i386_linux_kernel_vmlinux_head[0],
         sizeof(stub_i386_linux_kernel_vmlinux_head)-(1+ 4) +1);
-    U32 tmp_u32; tmp_u32 = ph.c_len; fo->write(&tmp_u32, 4);
+    TE32 tmp_u32; tmp_u32 = ph.c_len; fo->write(&tmp_u32, 4);
 
     stxt->sh_size += sizeof(stub_i386_linux_kernel_vmlinux_head);
 
@@ -1032,7 +1029,7 @@ unsigned PackVmlinuxAMD64::write_vmlinux_head(
     // COMPRESSED_LENGTH
     fo->write(&stub_amd64_linux_kernel_vmlinux_head[0],
         sizeof(stub_amd64_linux_kernel_vmlinux_head)-(1+ 4) +1);
-    U32 tmp_u32; tmp_u32 = ph.c_len; fo->write(&tmp_u32, 4);
+    TE32 tmp_u32; tmp_u32 = ph.c_len; fo->write(&tmp_u32, 4);
 printf("  Compressed length=0x%x\n", ph.c_len);
 printf("UnCompressed length=0x%x\n", ph.u_len);
 
@@ -1091,7 +1088,7 @@ unsigned PackVmlinuxARMEL::write_vmlinux_head(
     fo->write(&stub_arm_linux_kernel_vmlinux_head[0], 4);
 
     // Second word
-    U32 tmp_u32;
+    TE32 tmp_u32;
     unsigned const t = (0xff000000 &
             BeLePolicy::get32(&stub_arm_linux_kernel_vmlinux_head[4]))
         | (0x00ffffff & (0u - 1 + ((3+ ph.c_len)>>2)));
@@ -1113,7 +1110,7 @@ unsigned PackVmlinuxARMEB::write_vmlinux_head(
     fo->write(&stub_armeb_linux_kernel_vmlinux_head[0], 4);
 
     // Second word
-    U32 tmp_u32;
+    TE32 tmp_u32;
     unsigned const t = (0xff000000 &
             BeLePolicy::get32(&stub_armeb_linux_kernel_vmlinux_head[4]))
         | (0x00ffffff & (0u - 1 + ((3+ ph.c_len)>>2)));
@@ -1137,7 +1134,7 @@ unsigned PackVmlinuxPPC32::write_vmlinux_head(
 
 bool PackVmlinuxARMEL::has_valid_vmlinux_head()
 {
-    U32 buf[2];
+    TE32 buf[2];
     fi->seek(p_text->sh_offset + sizeof(stub_arm_linux_kernel_vmlinux_head) -8, SEEK_SET);
     fi->readx(buf, sizeof(buf));
     //unsigned const word0 = buf[0];
@@ -1151,7 +1148,7 @@ bool PackVmlinuxARMEL::has_valid_vmlinux_head()
 
 bool PackVmlinuxARMEB::has_valid_vmlinux_head()
 {
-    U32 buf[2];
+    TE32 buf[2];
     fi->seek(p_text->sh_offset + sizeof(stub_armeb_linux_kernel_vmlinux_head) -8, SEEK_SET);
     fi->readx(buf, sizeof(buf));
     //unsigned const word0 = buf[0];
@@ -1165,7 +1162,7 @@ bool PackVmlinuxARMEB::has_valid_vmlinux_head()
 
 bool PackVmlinuxPPC32::has_valid_vmlinux_head()
 {
-    U32 buf[2];
+    TE32 buf[2];
     fi->seek(p_text->sh_offset + sizeof(stub_powerpc_linux_kernel_vmlinux_head) -8, SEEK_SET);
     fi->readx(buf, sizeof(buf));
     //unsigned const word0 = buf[0];
@@ -1392,8 +1389,8 @@ bool PackVmlinuxAMD64::has_valid_vmlinux_head()
 //      pushl $0
 //      pushl $ MAP_PRIVATE | MAP_ANONYMOUS | MAP_FIXED
 //      pushl $ PROT_EXEC | PROT_WRITE | PROT_READ
-//      pushl $0x600000  # 6MB length
-//      pushl $0x100000  # 1MB address
+//      pushl $0x600000  # 6 MiB length
+//      pushl $0x100000  # 1 MiB address
 //      call mmap
 //      leal -0x9000(%esp),%esi  # expect "lea 0x9000(%esi),%esp" later
 ///* Fall into .text of upx-compressed vmlinux. */
@@ -1442,7 +1439,7 @@ const int *PackVmlinuxAMD64::getFilters() const
 
 bool PackVmlinuxAMD64::is_valid_e_entry(Addr e_entry)
 {
-    return 0x200000<=e_entry; // 2MB
+    return 0x200000<=e_entry; // 2 MiB
 }
 
 Linker* PackVmlinuxAMD64::newLinker() const
