@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2007 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2007 Laszlo Molnar
+   Copyright (C) 1996-2009 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2009 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -21,8 +21,8 @@
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-   Markus F.X.J. Oberhumer   Laszlo Molnar
-   markus@oberhumer.com      ml1050@users.sourceforge.net
+   Markus F.X.J. Oberhumer              Laszlo Molnar
+   <markus@oberhumer.com>               <ml1050@users.sourceforge.net>
  */
 
 
@@ -45,6 +45,7 @@ public:
     virtual const char *getFullName(const options_t *) const { return "i386-linux.kernel.vmlinuz"; }
     virtual const int *getCompressionMethods(int method, int level) const;
     virtual const int *getFilters() const;
+    virtual int getStrategy(Filter &);
 
     virtual void pack(OutputFile *fo);
     virtual void unpack(OutputFile *fo);
@@ -63,13 +64,12 @@ protected:
 //    virtual upx_byte *getLoader() const;
 //    virtual int getLoaderSize() const;
 
-    struct boot_sect_t
-    {
+    __packed_struct(boot_sect_t)
         char            _[0x1f1];
         unsigned char   setup_sects;
         char            __[2];
-        LE16            sys_size;
-        char            ___[8];
+        LE32            sys_size;
+        char            ___[6];
         LE16            boot_flag;      // 0xAA55
         // 0x200
         char            ____[2];
@@ -79,15 +79,24 @@ protected:
         unsigned char   load_flags;
         char            ______[2];
         LE32            code32_start;
-
+        char            _7[0x230 - (0x214 + 4)];
+        LE32            kernel_alignment;
+        char            relocatable_kernel;
+        char            _8[0x248 - (0x234 + 1)];
+        LE32            payload_offset;
+        LE32            payload_length;
         // some more uninteresting fields here ...
-        // see /usr/src/linux/Documentation/i386/zero-page.txt
-    }
-    __attribute_packed;
+        // see /usr/src/linux/Documentation/i386/boot.txt
+    __packed_struct_end()
+
+    boot_sect_t h;
 
     MemBuffer setup_buf;
     int setup_size;
     unsigned physical_start;
+    unsigned page_offset;
+    unsigned config_physical_align;
+    unsigned filter_len;
 };
 
 
@@ -103,6 +112,7 @@ public:
     virtual int getFormat() const { return UPX_F_BVMLINUZ_i386; }
     virtual const char *getName() const { return "bvmlinuz/386"; }
     virtual const char *getFullName(const options_t *) const { return "i386-linux.kernel.bvmlinuz"; }
+    virtual const int *getFilters() const;
 
     virtual void pack(OutputFile *fo);
 
@@ -139,6 +149,52 @@ protected:
 };
 
 
+/*************************************************************************
+// vmlinuz/armel (gzip compressed Linux kernel image)
+**************************************************************************/
+
+class PackVmlinuzARMEL : public Packer
+{
+    typedef Packer super;
+public:
+    PackVmlinuzARMEL(InputFile *f);
+    virtual int getVersion() const { return 13; }
+    virtual int getFormat() const { return UPX_F_VMLINUZ_ARMEL; }
+    virtual const char *getName() const { return "vmlinuz/armel"; }
+    virtual const char *getFullName(const options_t *) const { return "armel-linux.kernel.vmlinuz"; }
+    virtual const int *getCompressionMethods(int method, int level) const;
+    virtual const int *getFilters() const;
+    virtual int getStrategy(Filter &);
+
+    virtual void pack(OutputFile *fo);
+    virtual void unpack(OutputFile *fo);
+
+    virtual bool canPack();
+    virtual int canUnpack();
+
+protected:
+    virtual int readFileHeader();
+    virtual int decompressKernel();
+    virtual void readKernel();
+
+    virtual void buildLoader(const Filter *ft);
+    virtual unsigned write_vmlinuz_head(OutputFile *const fo);
+    virtual void defineDecompressorSymbols();
+    virtual Linker* newLinker() const;
+
+//    virtual upx_byte *getLoader() const;
+//    virtual int getLoaderSize() const;
+
+
+    MemBuffer setup_buf;
+    int setup_size;
+//    unsigned physical_start;
+//    unsigned page_offset;
+//    unsigned config_physical_align;
+    unsigned filter_len;
+};
+
+
 #endif /* already included */
 
 
