@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2007 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2007 Laszlo Molnar
+   Copyright (C) 1996-2009 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2009 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -22,7 +22,7 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    Markus F.X.J. Oberhumer              Laszlo Molnar
-   <mfx@users.sourceforge.net>          <ml1050@users.sourceforge.net>
+   <markus@oberhumer.com>               <ml1050@users.sourceforge.net>
  */
 
 
@@ -164,7 +164,7 @@ int PackW32Pe::readFileHeader()
     char buf[6];
     fi->seek(0x200, SEEK_SET);
     fi->readx(buf, 6);
-    isrtm = memcmp(&buf, "32STUB" ,6) == 0;
+    isrtm = memcmp(buf, "32STUB" ,6) == 0;
     return super::readFileHeader();
 }
 
@@ -173,14 +173,12 @@ int PackW32Pe::readFileHeader()
 // import handling
 **************************************************************************/
 
-struct import_desc
-{
+__packed_struct(import_desc)
     LE32  oft;      // orig first thunk
     char  _[8];
     LE32  dllname;
     LE32  iat;      // import address table
-}
-__attribute_packed;
+__packed_struct_end()
 
 void PackW32Pe::processImports(unsigned myimport, unsigned) // pass 2
 {
@@ -1037,7 +1035,7 @@ void PackW32Pe::pack(OutputFile *fo)
     //    set_le32(ibuf + ic,get_le32("UPX "));
     ibuf.clear(0, oh.filealign);
 
-    info("Image size change: %u -> %u kBytes",
+    info("Image size change: %u -> %u KiB",
          ih.imagesize / 1024, oh.imagesize / 1024);
 
     infoHeader("[Writing compressed file]");
@@ -1105,7 +1103,7 @@ void PackW32Pe::pack(OutputFile *fo)
 
 int PackW32Pe::canUnpack()
 {
-    if (!readFileHeader())
+    if (!readFileHeader() || ih.cpu < 0x14c || ih.cpu > 0x150)
         return false;
 
     unsigned objs = ih.objects;
