@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2007 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2007 Laszlo Molnar
+   Copyright (C) 1996-2009 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2009 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -21,8 +21,8 @@
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-   Markus F.X.J. Oberhumer   Laszlo Molnar
-   markus@oberhumer.com      ml1050@users.sourceforge.net
+   Markus F.X.J. Oberhumer              Laszlo Molnar
+   <markus@oberhumer.com>               <ml1050@users.sourceforge.net>
  */
 
 
@@ -240,6 +240,7 @@ protected:
 #else
     void __acc_cdecl_va addLoaderVA(const char *s, ...);
 #endif
+    virtual bool hasLoaderSection(const char *name) const;
     virtual int getLoaderSection(const char *name, int *slen=NULL) const;
     virtual int getLoaderSectionStart(const char *name, int *slen=NULL) const;
 
@@ -283,8 +284,16 @@ protected:
     static upx_byte *optimizeReloc32(upx_byte *in,unsigned relocnum,upx_byte *out,upx_byte *image,int bs,int *big);
     static unsigned unoptimizeReloc32(upx_byte **in,upx_byte *image,MemBuffer *out,int bs);
 
+    // target endianness abstraction
+    unsigned get_te16(const void *p)        const { return bele->get16(p); }
+    unsigned get_te32(const void *p)        const { return bele->get32(p); }
+    acc_uint64l_t get_te64(const void *p)   const { return bele->get64(p); }
+    void set_te16(void *p, unsigned v)      const { bele->set16(p, v); }
+    void set_te32(void *p, unsigned v)      const { bele->set32(p, v); }
+    void set_te64(void *p, acc_uint64l_t v) const { bele->set64(p, v); }
+
 protected:
-    const N_BELE_RTP::AbstractPolicy *bele;
+    const N_BELE_RTP::AbstractPolicy *bele; // target endianness
     InputFile *fi;
     off_t file_size;        // will get set by constructor
     PackHeader ph;          // must be filled by canUnpack()
