@@ -2,9 +2,9 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2007 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2007 Laszlo Molnar
-   Copyright (C) 2000-2007 John F. Reiser
+   Copyright (C) 1996-2009 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2009 Laszlo Molnar
+   Copyright (C) 2000-2009 John F. Reiser
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -23,7 +23,7 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    Markus F.X.J. Oberhumer              Laszlo Molnar
-   <mfx@users.sourceforge.net>          <ml1050@users.sourceforge.net>
+   <markus@oberhumer.com>               <ml1050@users.sourceforge.net>
 
    John F. Reiser
    <jreiser@users.sourceforge.net>
@@ -371,7 +371,7 @@ umax(unsigned a, unsigned b)
 // OpenBSD 3.9 wants all .text from all modules together, then (above that)
 // all .data from all modules together, in order to maximize the effectiveness
 // of lowering the segment limit on %cs as an implementation of "no-execute
-// .data".  Thus OpenBSD 3.9 puts a gap of 0x20000000 (512MB) between the
+// .data".  Thus OpenBSD 3.9 puts a gap of 0x20000000 (512 MiB) between the
 // origins of .text and .data in each module.  So, mapping ET_DYN must "level"
 // the excursion in .text and .data of anything which preceeded it,
 // then add the new PT_LOAD.
