@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2007 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2007 Laszlo Molnar
+   Copyright (C) 1996-2009 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2009 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -22,7 +22,7 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    Markus F.X.J. Oberhumer              Laszlo Molnar
-   <mfx@users.sourceforge.net>          <ml1050@users.sourceforge.net>
+   <markus@oberhumer.com>               <ml1050@users.sourceforge.net>
  */
 
 
@@ -103,6 +103,7 @@ struct timespec {
 
 #define SEEK_SET        0
 #define SEEK_CUR        1
+#define SEEK_END        2
 
 #define O_RDONLY        00
 #define O_WRONLY        01
@@ -156,6 +157,7 @@ struct timespec {
 #define __NR_open                 5
 #define __NR_close                6
 #define __NR_waitpid              7
+#define __NR_link                 9
 #define __NR_unlink              10
 #define __NR_execve              11
 #define __NR_lseek               19
@@ -347,6 +349,7 @@ static inline _syscall3(ssize_t,read,int,fd,void *,buf,size_t,count)
 static inline _syscall3(pid_t,waitpid,pid_t,pid,int *,wait_stat,int,options)
 static inline _syscall3(ssize_t,write,int,fd,const void *,buf,size_t,count)
 static inline _syscall1(int,unlink,const char *,file)
+static inline _syscall2(int,link,const char *,src, const char *,dst)
 
 #undef Z0
 #undef Z1
